import { getInputDir, getOutputDir, makeDirSync } from "./env";
import { randomCode } from "./utils";
const fs = require("fs");
const path = require("path");

/**
 * 旋转图片，并保存为正式文件
 * @param {*} imgPath 图片路径
 * @param {String} paperInfo 保持文件名称
 * @param {Object} collectConfig 裁剪区域
 */
async function saveOutputImage(imgPath, paperInfo, collectConfig) {
  // console.log(collectConfig);
  const outputOriginPath = saveOriginImage(imgPath, paperInfo);

  return outputOriginPath;
}

function saveOriginImage(imgPath, paperInfo) {
  if (paperInfo.compressRate === 100) return "";

  const outputOriginPath = getOutputImagePath(paperInfo, "origin");
  fs.copyFileSync(imgPath, outputOriginPath);
  return outputOriginPath;
}

function getOutputImagePath(paperInfo, type) {
  const outputDir = path.join(
    getOutputDir(type),
    paperInfo.examId + "",
    paperInfo.subjectId + ""
  );

  if (!fs.existsSync(outputDir)) makeDirSync(outputDir);
  return path.join(outputDir, `${paperInfo.examNumber}-${randomCode()}.jpg`);
}

/**
 * 获取最早添加的文件
 * @param {String} dir 图片目录
 */
function getEarliestFile(dir) {
  const ddir = dir || getInputDir();
  const files = fs
    .readdirSync(ddir)
    .filter(fileName => fileName.toLowerCase().match(/\.(jpg|png|jpeg)/))
    .map(fileName => {
      return {
        name: fileName,
        time: fs.statSync(path.join(ddir, fileName)).birthtimeMs
      };
    })
    .sort((a, b) => a.time - b.time);

  if (!files.length) return { url: "", name: "" };

  return {
    url: path.join(ddir, files[0].name),
    name: files[0].name
  };
}

export { saveOutputImage, getEarliestFile };
